import cv2
import matplotlib.pyplot as plt
import cloudproc
import os
import tensorflow as tf
import numpy as np
import datetime
from cloudproc.config import DATASET_DIR, TRAIN_FILE
from cloudproc.cloud_cnn import _parse_function
from cloudproc.reading.sunlocation import getSunAltitude



def showImgsInDataSet(train_filename, sample=200):
    sample_imgs = []
    record_iterator = tf.python_io.tf_record_iterator(path=train_filename)
    for i, string_record in enumerate(record_iterator):
        if i > sample: break
        example = tf.train.Example()
        example.ParseFromString(string_record)
        def int64Feat(name):
            return int(example.features.feature[name].int64_list.value[0])
        def floatFeat(name):
            return float(example.features.feature[name].float_list.value[0])
        height = int64Feat('height')
        width = int64Feat('width')
        height = int64Feat('height')

        height_reading = floatFeat('height_reading')
        rain_detection = int64Feat('rain_detection')
        timestamp = int64Feat('timestamp')
        
        img_string = example.features.feature['image_encoded'].bytes_list.value[0]

        imgBGR = cv2.imdecode(np.fromstring(img_string, dtype=np.uint8), 1)
        imgRGB = cv2.cvtColor(imgBGR, cv2.COLOR_BGR2RGB)
        sample_imgs.append((imgRGB, height_reading, datetime.datetime.utcfromtimestamp(timestamp), rain_detection))

    columns = int(np.ceil(np.sqrt(len(sample_imgs))))
    rows = int(np.ceil(len(sample_imgs) / columns))

    plt.figure(figsize = (rows,columns))

    for i, img in enumerate(sample_imgs):
        ax = plt.subplot(rows, columns, i+1)
        alt = getSunAltitude(img[2])
        ax.set(title='{0[3]}[H:{0[1]:.2f}] {1:.2f} \n {2}'.format(img, alt, img[2].strftime('%d%m%Y;%H:%M:%S')))
        plt.axis('off')
        plt.imshow(img[0])
    plt.show()


train_file = 'train_small.tfrecords'
validation_file = 'validation_small.tfrecords'
cloudproc.tf_records.saveTrainAndValidationSet(sample_size=50, train=train_file, validation=validation_file,\
 image_dir='..\\cloud_images', ceilometer_dir='..\\ct')

train_filename = os.path.join(DATASET_DIR, train_file)
showImgsInDataSet(train_filename)

#cloudproc.cloud_cnn.main(None)