import tensorflow as tf
import cv2
import numpy as np
import os

from cloudproc.cloud_cnn import heights_to_labels_np, num_height_classes
from cloudproc.validate import load_heightpoints, height_histo, random_heights
from cloudproc.config import HEIGHT_BINS, DATASET_DIR, TRAIN_FILE, VALIDATION_FILE



def collect_class_samples(train_filename, min_samples_per_class=7):
    samples = {}
    record_iterator = tf.python_io.tf_record_iterator(path=train_filename)
    for string_record in record_iterator:
        # check if done
        if all(i in samples and len(samples[(r, i)]) >= min_samples_per_class for i in range(num_height_classes()) for r in range(1)):
            break

        example = tf.train.Example()
        example.ParseFromString(string_record)
        def floatFeat(name):
            return float(example.features.feature[name].float_list.value[0])
        def intFeat(name):
            return float(example.features.feature[name].int64_list.value[0])
        rain_detection = intFeat('rain_detection')
        height_reading = floatFeat('height_reading')
        img_string = example.features.feature['image_encoded'].bytes_list.value[0]

        imgBGR = cv2.imdecode(np.fromstring(img_string, dtype=np.uint8), 1)
        val = (height_reading, imgBGR)
        lbl = (rain_detection, heights_to_labels_np(np.array([height_reading])).sum())
        if lbl in samples:
            samples[lbl].append(val)
        else: samples[lbl] = [val]
    return samples


def mergeImages(samples, rain=False):
    vis_img = None
    for r, lbl in sorted(samples):
        if r != rain:
            continue
        img_list = samples[(r, lbl)]
        print('label', lbl)
        print('rain', r)
        # sort by height_reading
        sorted_imgs = sorted(img_list, key=lambda v:v[0])
        heights = [h for h,img in sorted_imgs]
        imgs = [img for h, img in sorted_imgs]
        print('heights', heights)
        # merge next to one another
        row_img = np.concatenate(imgs[:min_count], axis=1)
        if vis_img is not None:
            # stack on top of one another
            vis_img = np.concatenate((vis_img, row_img), axis=0)
        else: vis_img = row_img
    return vis_img


train_filename = os.path.join(DATASET_DIR, TRAIN_FILE)

min_count = 7
samples = collect_class_samples(train_filename, min_count)


cv2.imwrite('class_visualization_norain.png', mergeImages(samples, False))
cv2.imwrite('class_visualization_rain.png', mergeImages(samples, True))
